﻿'use strict';

var argv = require('minimist')(process.argv.slice(2));
var fs = require('fs');
var pathlib = require('path');
var vinylFile = require('vinyl-file');
var glob = require("glob");
var chalk = require('chalk');
var extend = require('extend');

var defaultSettings = {
  "projects": [
    {
      "name": "default",
      "projectFolder": "",
      "globPattern": './**/*.tpl.html',
      "outFolder": null
    }
  ]
};

var html2ts = function () {
  if (!argv) {
    return;
  }

  var html2tsSettingsFilePath = pathlib.join(pathlib.dirname(__filename), '../../', 'html2ts.json');
  var projectNames = argv.p;

  if (projectNames && projectNames.constructor !== Array) {
    projectNames = [ projectNames ];
  }

  vinylFile.read(html2tsSettingsFilePath).then(file => {
    console.info(chalk.yellowBright('Using html2ts.json settings file. Path: ' + html2tsSettingsFilePath));
    var fileSettings = JSON.parse(file.contents.toString());
    init(argv.b, argv.w, projectNames, fileSettings);
  }).catch((err) => {
    //console.log(err);
    console.info(chalk.yellowBright('No html2ts.json settings file... using defaults. Path: ' + html2tsSettingsFilePath));
    init(argv.b, argv.w, projectNames, defaultSettings);
  })
};

function init(build, watch, projectNames, settings) {
  for (let i = 0; i < settings.projects.length; i++) {
    if (shouldRun(projectNames, settings.projects[i].name)) {
  if (argv.b) {
        buidCore(settings.projects[i]);
  }
  if (argv.w) {
        watchCore(settings.projects[i]);
  }
}
  }
}

function shouldRun(projectNames, projectName) {
  if (!projectNames) {
    return true;
  }

  if (projectNames.length < 1) {
    return true;
  }

  for (var i = 0; i < projectNames.length; i++) {
    if (projectNames[i] === projectName) {
      return true;
    }
  }

  return false;
}

function buidCore(settings) {
  var fileGlob = settings.projectFolder + "/" + settings.globPattern;

  console.info(chalk.magentaBright("Building all " + fileGlob));
  glob(fileGlob, function (er, files) {
    for (var i = 0; i < files.length; i++) {
      console.info("Building: " + files[i]);
      createHtml(files[i], settings);
    }
  });
}

function watchCore(settings) {
  var watch = require('glob-watcher');
  var fileGlob = settings.projectFolder + "/" + settings.globPattern;
  var watcher = watch(fileGlob);
  console.info("Watching " + fileGlob);
  watcher.on('change', function (path, stat) {
    createHtml(path, settings);
  });
}

function createHtml(path, settings) {
  vinylFile.read(path).then(file => {
    if (file.isNull()) {
      return;
    }

    var fileName = pathlib.basename(file.path, '.html');

    if (!fileName) {
      console.error('file <' + file.path + '> not converted!');
      return done();
    }

    var content = stripBOM(escapeContent(file.contents.toString()));

    content = "module $fileName { export var html = \'$fileContent\';}"
      .replace("$fileName", fileName)
      .replace('$fileContent', content);

    var outFile = getOuputPath(file, settings);

    fs.writeFile(outFile, content, (err) => {
      if (err) {
        throw err;
      }

      console.log(chalk.green("Wrote file: '" + outFile + "'"));
    });
  });
      }

function getOuputPath(file, settings) {
  var outFile = file.path + '.ts';

  if (settings.outFolder) {
    var outDir = settings.projectFolder + "/" + settings.outFolder;

      if (!fs.existsSync(outDir)) {
        fs.mkdirSync(outDir);
      }

      outFile = pathlib.join(outDir, pathlib.basename(file.path) + '.ts');
    }

  return outFile;
      }


// html -> js processing functions:
// Originally from karma-html2js-preprocessor
function escapeContent(content, quoteChar) {
  quoteChar = quoteChar || '\'';

  var quoteRegexp = new RegExp('\\' + quoteChar, 'g');
  var nlReplace = '';

  return content.replace(quoteRegexp, '\\' + quoteChar).replace(/\r?\n/g, nlReplace);
};

// Remove bom when reading utf8 files
function stripBOM(str) {
  return 0xFEFF === str.charCodeAt(0) ? str.substring(1) : str;
}

module.exports = html2ts;